@@ -12,7 +12,6 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #include <QAction>
 #include <QFile>
 #include <QDir>
-#include <QRegExp>
 #include <QTextCodec>
 #include <QTextStream>
 #include <QList>
@@ -64,11 +63,11 @@ KompareModelList::KompareModelList(DiffSettings* diffSettings, QWidget* widgetFo
         m_applyAll = m_actionCollection->addAction(QStringLiteral("difference_applyall"), this, &KompareModelList::slotActionApplyAllDifferences);
         m_applyAll->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right-double")));
         m_applyAll->setText(i18nc("@action", "App&ly All"));
-        m_actionCollection->setDefaultShortcut(m_applyAll, QKeySequence(Qt::CTRL + Qt::Key_A));
+        m_actionCollection->setDefaultShortcut(m_applyAll, QKeySequence(Qt::CTRL | Qt::Key_A));
         m_unapplyAll = m_actionCollection->addAction(QStringLiteral("difference_unapplyall"), this, &KompareModelList::slotActionUnapplyAllDifferences);
         m_unapplyAll->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left-double")));
         m_unapplyAll->setText(i18nc("@action", "&Unapply All"));
-        m_actionCollection->setDefaultShortcut(m_unapplyAll, QKeySequence(Qt::CTRL + Qt::Key_U));
+        m_actionCollection->setDefaultShortcut(m_unapplyAll, QKeySequence(Qt::CTRL | Qt::Key_U));
     } else {
         m_applyDifference = nullptr;
         m_unApplyDifference = nullptr;
@@ -78,19 +77,19 @@ KompareModelList::KompareModelList(DiffSettings* diffSettings, QWidget* widgetFo
     m_previousFile = m_actionCollection->addAction(QStringLiteral("difference_previousfile"), this, &KompareModelList::slotPreviousModel);
     m_previousFile->setIcon(QIcon::fromTheme(QStringLiteral("arrow-up-double")));
     m_previousFile->setText(i18nc("@action", "P&revious File"));
-    m_actionCollection->setDefaultShortcut(m_previousFile, QKeySequence(Qt::CTRL + Qt::Key_PageUp));
+    m_actionCollection->setDefaultShortcut(m_previousFile, QKeySequence(Qt::CTRL | Qt::Key_PageUp));
     m_nextFile = m_actionCollection->addAction(QStringLiteral("difference_nextfile"), this, &KompareModelList::slotNextModel);
     m_nextFile->setIcon(QIcon::fromTheme(QStringLiteral("arrow-down-double")));
     m_nextFile->setText(i18nc("@action", "N&ext File"));
-    m_actionCollection->setDefaultShortcut(m_nextFile, QKeySequence(Qt::CTRL + Qt::Key_PageDown));
+    m_actionCollection->setDefaultShortcut(m_nextFile, QKeySequence(Qt::CTRL | Qt::Key_PageDown));
     m_previousDifference = m_actionCollection->addAction(QStringLiteral("difference_previous"), this, &KompareModelList::slotPreviousDifference);
     m_previousDifference->setIcon(QIcon::fromTheme(QStringLiteral("arrow-up")));
     m_previousDifference->setText(i18nc("@action", "&Previous Difference"));
-    m_actionCollection->setDefaultShortcut(m_previousDifference, QKeySequence(Qt::CTRL + Qt::Key_Up));
+    m_actionCollection->setDefaultShortcut(m_previousDifference, QKeySequence(Qt::CTRL | Qt::Key_Up));
     m_nextDifference = m_actionCollection->addAction(QStringLiteral("difference_next"), this, &KompareModelList::slotNextDifference);
     m_nextDifference->setIcon(QIcon::fromTheme(QStringLiteral("arrow-down")));
     m_nextDifference->setText(i18nc("@action", "&Next Difference"));
-    m_actionCollection->setDefaultShortcut(m_nextDifference, QKeySequence(Qt::CTRL + Qt::Key_Down));
+    m_actionCollection->setDefaultShortcut(m_nextDifference, QKeySequence(Qt::CTRL | Qt::Key_Down));
     m_previousDifference->setEnabled(false);
     m_nextDifference->setEnabled(false);
 
@@ -388,14 +387,14 @@ bool KompareModelList::saveDestination(DiffModel* model)
         qCDebug(LIBKOMPAREDIFF2) << "Tempfilename (save) : " << temp.fileName();
         qCDebug(LIBKOMPAREDIFF2) << "Model->path+file    : " << model->destinationPath() << model->destinationFile();
         qCDebug(LIBKOMPAREDIFF2) << "info->localdest     : " << m_info->localDestination;
-        QString tmp = model->destinationPath() + model->destinationFile();
+        QString tmp = model->destinationPath();
         if (tmp.startsWith(m_info->localDestination))     // It should, if not serious trouble...
             tmp.remove(0, m_info->localDestination.size());
         qCDebug(LIBKOMPAREDIFF2) << "DestinationURL      : " << m_info->destination;
         qCDebug(LIBKOMPAREDIFF2) << "tmp                 : " << tmp;
         KIO::UDSEntry entry;
         QUrl fullDestinationPath = m_info->destination;
-        fullDestinationPath.setPath(fullDestinationPath.path() + QLatin1Char('/') + tmp);
+        fullDestinationPath.setPath(fullDestinationPath.path() + tmp);
         qCDebug(LIBKOMPAREDIFF2) << "fullDestinationPath : " << fullDestinationPath;
         KIO::StatJob* statJob = KIO::stat(fullDestinationPath);
         if (!statJob->exec())
@@ -408,6 +407,7 @@ bool KompareModelList::saveDestination(DiffModel* model)
                 return false;
             }
         }
+        fullDestinationPath.setPath(fullDestinationPath.path() + model->destinationFile());
         KIO::FileCopyJob* copyJob = KIO::file_copy(QUrl::fromLocalFile(temp.fileName()), fullDestinationPath, -1, KIO::Overwrite);
         result = copyJob->exec();
     }
@@ -415,7 +415,15 @@ bool KompareModelList::saveDestination(DiffModel* model)
     {
         qCDebug(LIBKOMPAREDIFF2) << "Tempfilename   : " << temp.fileName();
         qCDebug(LIBKOMPAREDIFF2) << "DestinationURL : " << m_info->destination;
-        KIO::FileCopyJob* copyJob = KIO::file_copy(QUrl::fromLocalFile(temp.fileName()), m_info->destination , -1, KIO::Overwrite);
+
+        // Get permissions of existing file and copy temporary file with the same permissions
+        int permissions = -1;
+        KIO::StatJob* statJob = KIO::stat(m_info->destination);
+        result = statJob->exec();
+        if (result)
+            permissions = statJob->statResult().numberValue(KIO::UDSEntry::UDS_ACCESS);
+
+        KIO::FileCopyJob* copyJob = KIO::file_copy(QUrl::fromLocalFile(temp.fileName()), m_info->destination , permissions, KIO::Overwrite);
         result = copyJob->exec();
         qCDebug(LIBKOMPAREDIFF2) << "true or false?" << result;
     }
@@ -585,15 +593,17 @@ QString KompareModelList::readFile(const QString& fileName)
     QFile file(fileName);
     file.open(QIODevice::ReadOnly);
 
-    QTextStream stream(&file);
     qCDebug(LIBKOMPAREDIFF2) << "Codec = " << m_textCodec;
-
     if (!m_textCodec)
         m_textCodec = QTextCodec::codecForLocale();
+    std::unique_ptr<QTextDecoder> decoder(m_textCodec->makeDecoder());
 
-    stream.setCodec(m_textCodec);
-
-    QString contents = stream.readAll();
+    QString contents;
+    while (!file.atEnd()) {
+        char buffer[4096];
+        const auto len = file.read(buffer, 4096);
+        contents += decoder->toUnicode(buffer, len);
+    }
 
     file.close();
 
