@@ -8,7 +8,7 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #ifndef DIFF2_PARSERBASE_H
 #define DIFF2_PARSERBASE_H
 
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QStringList>
 
 #include "kompare.h"
@@ -63,7 +63,7 @@ protected:
 
 protected: // Helper methods to speed things up
     bool matchesUnifiedHunkLine(const QString& line) const;
-    void checkHeader(const QRegExp& header);
+    void checkHeader(const QRegularExpression& header);
 
 protected:
     /** What is format of the diff */
@@ -71,45 +71,46 @@ protected:
 
 protected:
     // Regexps for context parsing
-    QRegExp m_contextDiffHeader1;
-    QRegExp m_contextDiffHeader2;
+    QRegularExpression m_contextDiffHeader1;
+    QRegularExpression m_contextDiffHeader2;
 
-    QRegExp m_contextHunkHeader1;
-    QRegExp m_contextHunkHeader2;
-    QRegExp m_contextHunkHeader3;
+    QRegularExpression m_contextHunkHeader1;
+    QRegularExpression m_contextHunkHeader2;
+    QRegularExpression m_contextHunkHeader3;
+    QRegularExpressionMatch m_contextHunkHeader1Match;
+    QRegularExpressionMatch m_contextHunkHeader2Match;
 
-    QRegExp m_contextHunkBodyRemoved;
-    QRegExp m_contextHunkBodyAdded;
-    QRegExp m_contextHunkBodyChanged;
-    QRegExp m_contextHunkBodyContext;
-    QRegExp m_contextHunkBodyLine; // Added for convenience
+    QRegularExpression m_contextHunkBodyRemoved;
+    QRegularExpression m_contextHunkBodyAdded;
+    QRegularExpression m_contextHunkBodyChanged;
+    QRegularExpression m_contextHunkBodyContext;
+    QRegularExpression m_contextHunkBodyLine; // Added for convenience
 
     // Regexps for normal parsing
-    QRegExp m_normalDiffHeader;
+    QRegularExpression m_normalDiffHeader;
 
-    QRegExp m_normalHunkHeaderAdded;
-    QRegExp m_normalHunkHeaderRemoved;
-    QRegExp m_normalHunkHeaderChanged;
+    QRegularExpression m_normalHunkHeaderAdded;
+    QRegularExpression m_normalHunkHeaderRemoved;
+    QRegularExpression m_normalHunkHeaderChanged;
+    QRegularExpressionMatch m_normalHunkHeaderAddedMatch;
+    QRegularExpressionMatch m_normalHunkHeaderRemovedMatch;
+    QRegularExpressionMatch m_normalHunkHeaderChangedMatch;
 
-    QRegExp m_normalHunkBodyRemoved;
-    QRegExp m_normalHunkBodyAdded;
-    QRegExp m_normalHunkBodyDivider;
+    QRegularExpression m_normalHunkBodyRemoved;
+    QRegularExpression m_normalHunkBodyAdded;
+    QRegularExpression m_normalHunkBodyDivider;
 
     enum Difference::Type m_normalDiffType;
 
     // RegExps for rcs parsing
-    QRegExp m_rcsDiffHeader;
+    QRegularExpression m_rcsDiffHeader;
 
     // Regexps for unified parsing
-    QRegExp m_unifiedDiffHeader1;
-    QRegExp m_unifiedDiffHeader2;
+    QRegularExpression m_unifiedDiffHeader1;
+    QRegularExpression m_unifiedDiffHeader2;
 
-    QRegExp m_unifiedHunkHeader;
-
-    QRegExp m_unifiedHunkBodyAdded;
-    QRegExp m_unifiedHunkBodyRemoved;
-    QRegExp m_unifiedHunkBodyContext;
-    QRegExp m_unifiedHunkBodyLine; // Added for convenience
+    QRegularExpression m_unifiedHunkHeader;
+    QRegularExpressionMatch m_unifiedHunkHeaderMatch;
 
 protected:
     const QStringList&         m_diffLines;
